# Brunch docs

Sections:

* [Guide](https://github.com/brunch/brunch-guide#readme)
* [FAQ](./faq.md), also tips on Bower integration
* [Commands](./commands.md)
* [Config](./config.md)
* [Plugins](./plugins.md)
* [Upgrading](./upgrading.md)

## Basics

Example application structure:

```
root/         # Main brunch application directory.
|-app/        # Your code may reside here.
|--assets/    # Static files that shall not be compiled
|---images/   # will be just copied to `public` dir.
|--views/     # Create any subdirectories inside `app` dir.
|---file-1.js # JS files will be automatically concatenated.
|--git.coffee # They will be also usable as modules, like require('git').
|--file-1.css # CSS files will be automatically concatenated too.
|--file-2.css # JS and CSS files may be linted before concatenation.
|--file.sass  # Compiled-to-css files will be concatenated to style file too.
|--tpl.jade   # You may have pre-compiled to JS templates. Also with `require`.

|-bower_components/ # Libraries, installed by Bower package manager
|--...stuff...      # will reside here if you use it.

|-vendor/     # All third-party libraries that are not handled by Bower
              # should be put here. JS, CSS, etc.
|--scripts/   # They will be included BEFORE your scripts automatically.
|---backbone.js
|---underscore.js

|-public/      # The directory with static files which is generated by brunch.
|--app.js      # Ready to be served via webserver.
|--app.css     # Don’t change it directly, just run `brunch watch --server`.
|--assets/     # And then all changed files in your app dir will be compiled.
|---images/

|-bower.json    # Contains info about Bower packages.
|-package.json  # Contains all brunch plugins, like jshint-brunch, css-brunch.
|-brunch-config.js # All params (you can concat to 2, 5, 10 files etc.)
                # are set via this config. Just simple, 15 lines-of-code config
```

### Concatenation

Brunch concatenates all your scripts in directories specified in
`config.paths.watched`. You set concatenation config, number of
output files in `config.files[type]`.

We suggest to have two files:

* `app.js` contains your application code.
* `vendor.js` contains code of libraries you depend on (e.g. jQuery).

This is better solution for browser caching than using one file,
because you change dependencies not as often as you change
your application code.

Order of file concatenation is:

1. Files in `config.files[type].order.before` in order you specify.
2. Bower components ordered automatically.
3. Files in `vendor/` directories in alphabetic order.
4. All other files in alphabetic order.
5. Files in `config.files[type].order.after` in order you specify.

All this stuff (conventions, name of out files etc) can be changed
via modifying config file.

### Conventions

Brunch also has conventions. Conventions are filters for files with special meaning. They can be changed via config.

* Static files in `assets/` dirs are copied directly to `public/`.
* Any scripts in `app/` dirs are wrapped by default into modules. These modules are Common.JS / AMD abstractions that allow you to simply get rid of global vars as to avoid polluting the global namespace. This is especially useful for larger projects. Any module that you need to use in the browser will have to be loaded with the `require('')` function. For example, if you have a script `app/views/user_view`, then you could load that in your html file using `<script>require('views/user_view')</script>`. File extensions are optional here.
* Scripts in `vendor/` dirs aren't wrapped in modules and as such do not require any further loading instructions.
* Files whose name start with `_` (underscore) are ignored by compiler. They're useful for languages like sass / stylus, where you import all substyles in main style file.
* Files named `-test.<extension>` are considered tests.
